///////////////////////////////////////////////////////////////////////////////
// Triplet class
//
// Last update: 29/06/2005
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;
import java.io.*;
import java.net.MalformedURLException;
import java.net.URL;
import java.applet.AudioClip;


///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.function.PuFunction;
import jv.number.PuDouble;
import jv.project.PjProject;
import jvx.loader.PgOffLoader;
import jvx.surface.PgDomainDescr;
import jvx.surface.PgParmSurface;
import jv.function.PuFunction;
import jv.geom.PgElementSet;
import jv.geom.PgPointSet;
import jv.geom.PgPolygon;
import jv.geom.PgPolygonSet;
import jvx.geom.PwCleanMesh;
import jvx.geom.PwClip;
import jvx.geom.PwModel;
import jvx.geom.PwUnfold;
import jv.number.PuDouble;
import jv.number.PuInteger;
import jv.object.PsDebug;
import jv.project.PgGeometryIf;
import jv.project.PgJvxSrc;
import jv.project.PjProject;
import jv.project.PvDisplayIf;
import jv.project.PvPickListenerIf;
import jv.project.PvPickEvent;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import jvx.geom.PwCleanMesh;
import jvx.loader.PgOffLoader;
import jvx.surface.PgDomainDescr;
import jvx.surface.PgParmSurface;


///////////////////////////////////////////////////////////////////////////////
// Triplet applet
///////////////////////////////////////////////////////////////////////////////
public class PjTriplet extends PjProject
{
   protected PgElementSet poly;
   protected URL codebase;
   protected URL fullpath;
   protected boolean showVertices = false;
   protected boolean showEdges = false;
   protected boolean showElements = true;
   
   protected double x_min = -9.0;
   protected double x_max = 9.0;
   protected double y_min = -9.0;
   protected double y_max = 9.0;
   protected double z_min = -9.0;
   protected double z_max = 9.0;   
   protected PgPolygonSet axes1;
   protected PgElementSet m_xplane;          // Our x = k plane
   protected PgElementSet m_yplane;          // Our y = k plane
   protected PgElementSet m_zplane;          // Our z = k plane
   
   protected AudioClip rightAnswerSoundData;
   protected AudioClip wrongAnswerSoundData;
   
   protected int level;
   protected int idiom;
   protected int length;                     // Game length

   
   //////////////////////////////////////////////////////////////////////////
   // Default constructor
   //////////////////////////////////////////////////////////////////////////
   public PjTriplet(URL input_codebase,
		            AudioClip input_rightAnswerSoundData,
		            AudioClip input_wrongAnswerSoundData,
		            int input_level,
		            int input_idiom,
			    int input_length)
   {
      super("Triplet Applet");
      
      codebase = input_codebase;
      rightAnswerSoundData = input_rightAnswerSoundData;
      wrongAnswerSoundData = input_wrongAnswerSoundData;
      level = input_level;
      idiom = input_idiom;
      length = input_length;

      if (getClass() == PjTriplet.class)
      {
         init();
      };
   }; // End of PjTriplet()


   //////////////////////////////////////////////////////////////////////////
   // Initialize variables
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
	   poly = new PgElementSet(3);
	   
       // Axes
       // Axes 1 configuration
       axes1 = new PgPolygonSet(3); // This is a 3D object
       axes1.setNumVertices(6);
       axes1.setNumPolygons(3);
       axes1.setVertex(0, new PdVector(x_min, 0.0, 0.0));
       axes1.setVertex(1, new PdVector(0.0, y_min, 0.0));
       axes1.setVertex(2, new PdVector(0.0, 0.0, z_min));
       axes1.setVertex(3, new PdVector(x_max, 0.0, 0.0));
       axes1.setVertex(4, new PdVector(0.0, y_max, 0.0));
       axes1.setVertex(5, new PdVector(0.0, 0.0, z_max));
       axes1.getVertex(0).setName("");
       axes1.getVertex(1).setName("");
       axes1.getVertex(2).setName("");
       axes1.getVertex(3).setName("x");
       axes1.getVertex(4).setName("y");
       axes1.getVertex(5).setName("z");
       axes1.showVertexLabels(true);
       axes1.setPolygon(0, 0, 3);
       axes1.setPolygon(1, 1, 4);
       axes1.setPolygon(2, 2, 5);
       axes1.setGlobalPolygonColor(Color.BLUE);
       axes1.setGlobalVertexColor(Color.BLUE);
       axes1.setPolygonSize(0, 0.5);
       axes1.setPolygonSize(1, 0.5);
       axes1.setPolygonSize(2, 0.5);
       axes1.showPolygonSizes(true);
       axes1.showPolygonEndArrow(true);
       
       // Define the three planes x = k, y = k and z = k
       m_xplane = new PgElementSet(3);
       m_xplane.setName("Cutting plane x = k");
       m_xplane.setNumElements(1);
       m_xplane.setNumVertices(4);
       m_xplane.setVertex(0, new PdVector(0, y_min, z_min));
       m_xplane.setVertex(1, new PdVector(0, y_max, z_min));
       m_xplane.setVertex(2, new PdVector(0, y_max, z_max));
       m_xplane.setVertex(3, new PdVector(0, y_min, z_max));
       m_xplane.setElement(0, 0, 1, 2, 3);

       m_yplane = new PgElementSet(3);
       m_yplane.setName("Cutting plane y = k");
       m_yplane.setNumElements(1);
       m_yplane.setNumVertices(4);
       m_yplane.setVertex(0, new PdVector(x_min, 0, z_min));
       m_yplane.setVertex(1, new PdVector(x_max, 0, z_min));
       m_yplane.setVertex(2, new PdVector(x_max, 0, z_max));
       m_yplane.setVertex(3, new PdVector(x_min, 0, z_max));
       m_yplane.setElement(0, 0, 1, 2, 3);

       m_zplane = new PgElementSet(3);
       m_zplane.setName("Cutting plane z = k");
       m_zplane.setNumElements(1);
       m_zplane.setNumVertices(4);
       m_zplane.setVertex(0, new PdVector(x_min, y_min, 0));
       m_zplane.setVertex(1, new PdVector(x_max, y_min, 0));
       m_zplane.setVertex(2, new PdVector(x_max, y_max, 0));
       m_zplane.setVertex(3, new PdVector(x_min, y_max, 0));
       m_zplane.setElement(0, 0, 1, 2, 3);
       
       m_xplane.setTransparency(0.7);
       m_xplane.showTransparency(true);
       m_xplane.setGlobalElementColor(Color.ORANGE);       
       m_yplane.setTransparency(0.7);
       m_yplane.showTransparency(true);
       m_yplane.setGlobalElementColor(Color.ORANGE);       
       m_zplane.setTransparency(0.7);
       m_zplane.showTransparency(true);
       m_zplane.setGlobalElementColor(Color.ORANGE);       
       
       
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Things to do when the class is started
   //////////////////////////////////////////////////////////////////////////
   public void start()
   {
	  addGeometry(poly);
	  
      update(this);
   }; // End of start()


   /////////////////////////////////////////////////////////////////////////
   // Update the class whenever a child has changed. Method is usually
   // invoked from the children.
   /////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (event == null)
      {
         return(true);
      };

      return(super.update(event));
   }; // End of update()
   
   //////////////////////////////////////////////////////////////////////////
   // Load off file
   //////////////////////////////////////////////////////////////////////////
   public void loadOffFile(String filename)
   {
      String pathname = codebase.toExternalForm() + filename;
      PgOffLoader offloader = new PgOffLoader();
      try
      {
         fullpath = new URL(pathname);
         java.io.InputStream r = fullpath.openStream();
         BufferedReader in = new BufferedReader(new InputStreamReader(r));
         if(in == null)
         {
            PsDebug.warning("could not open file  " + pathname + "...");
            System.out.println("could not open file  " + pathname + "...");
         }
	     else
         {
            // Loading polyhedron
            offloader.load(in);
            PgJvxSrc[] geometries;
            geometries = offloader.getGeometries();

	        // Configuring the polyhedron
	        poly.setJvx(geometries[0]);
            poly.showVertices(showVertices);
            poly.showElementColors(true);
            poly.showElements(showElements);
            poly.showEdges(showEdges);
            poly.showVertexOutline(true);
            poly.setGlobalEdgeColor(Color.BLACK);
            poly.setGlobalVertexColor(Color.WHITE);
            poly.showVertexColors(true);

            PwCleanMesh.identifyVertices(poly, 1E-010D);
            poly.makeNeighbour();
            poly.close();    
            
            // Center solid
            for (int i = 0; i < poly.getNumVertices(); i++)
            {
            	double x = poly.getVertex(i).getEntry(0);
            	double y = poly.getVertex(i).getEntry(1);
            	double z = poly.getVertex(i).getEntry(2);
            	poly.setVertex(i, x, y + 0.5, z + 0.5);
            }
         }
      }
      catch(MalformedURLException exp)
      {
         System.out.println(exp.toString());
      }
      catch(IOException exp)
      {
         System.out.println(exp.toString());
      }
   }; // End of loadOffFile()
   
}; // End of PjTriplet classs
